package AirOs;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(getDictWords makePassword cryptPass slurp loadConfig readProps writeProps makePsk popRoutes waitForPing defaultConfig channel2freq);

use strict;
use warnings;
use Data::Dumper;
use LWP::UserAgent;

my $DIC = '/usr/share/myspell/dicts/da_DK.dic';
my @DIC;

sub getDictWords {
    return @DIC if @DIC;
    
    open F, "<$DIC" or die "Unable to read $DIC: $!";
    my %seen;
    while (my $line = <F>) {
	chomp $line;
	$line =~ s!/.*$!!;
	next if length $line < 3;
	next if length $line > 5;
	next if $line !~ /^[a-z]+$/;
	$line = lc $line;
	next if $seen{$line}++;
	push @DIC, $line;
    }
    close F;
    
    @DIC = sort @DIC;
    return @DIC;
}

sub randomInt($) {
    my $max = shift;

    local *FILE;
    my $data;
    open (FILE, "</dev/urandom") or die "Couldn't read from /dev/urandom";
    read(FILE, $data, 4) or die "Couldn't read from /dev/urandom";
    close FILE;

    my $int = unpack('I',$data);

    return $int % $max;
}

sub makePassword() {
    my @words = getDictWords();
    return $words[randomInt(@words)] . randomInt(100) . $words[randomInt(@words)];
}

my @SALTY = ('.', '/', 0..9, 'A'..'Z', 'a'..'z');

sub cryptPass($) {
    my $pass = shift;

    return crypt($pass, join '', @SALTY[randomInt(@SALTY), randomInt(@SALTY)]); 
}

my @PSK = (0..9, 'A'..'Z', 'a'..'z');

sub makePsk {
    return join '', map {$PSK[randomInt(@PSK)]} (1..60); 
}

sub slurp($) {
    my $fn = shift;

    local $/ = undef;

    open F,"<$fn" or return undef;
    my $res = <F>;
    close F;

    return $res;
}

sub loadConfig($) {
    my ($cfgFile) = @_;
    
    my $cfgPerl = slurp($cfgFile) or die "Unable to read config from $cfgFile: $!";
    my $cfg = eval $cfgPerl;
    die "Error while evaling $cfgFile: $@" if $@;    

    return $cfg;
}

sub writeProps($$) {
    my ($fn, $props) = @_;
    
    open F, ">$fn" or die "Unable to write $fn: $!";
    for my $k (sort keys %$props) {
	die "cannot write null value for $k in $fn\n" unless defined $props->{$k};
	print F "$k=$props->{$k}\n";
    }
    close F;
}

sub readProps($) {
    my ($fn) = @_;

    my %props;
    open F, "<$fn" or die "Unable to read $fn: $!";
    while (my $line = <F>) {
	chomp $line;
	my ($k, $v) = $line =~ /^([^=]+?)\s*=\s*(.+)$/ or next;
	$props{$k} = $v;
    }
    close F;

    return \%props;
}

sub popRoutes($$$$);
sub popRoutes($$$$) {
    my ($cfg, $gw, $int, $pop) = @_;

    die "Invalid pop: $pop" unless $cfg->{pop}{$pop};

    # This pop net
    my @res = ({
	ip=>"10.$pop.0.0",
	netmask=>'255.255.0.0',	
	gateway=> $gw,
	devname=> $int,
    });

    # and all pops visible from here...
    for my $rds (sort {$a<=>$b} keys %{$cfg->{pop}{$pop}{ch}}) {
	next if $rds <= 15;	
	push @res, popRoutes($cfg, $gw, $int, $rds);
    }

    return @res;
}

sub waitForPing($) {
    my ($ip) = @_;

    print STDERR "Waiting for $ip to come online...";
    my $i = 0;
    while (`ping -w 10 -c 2 $ip` !~ / 2 received/) {
	print STDERR ".";
	$i++;
    }
    
    print STDERR "o\n";
    sleep 10 if $i;
    print STDERR "It's alive!!!\n";
}

my %CONFIGS = (

    sta=>{
	'aaa.1.status' => 'disabled',
	'aaa.1.wpa.key.1.mgmt' => 'WPA-PSK',
	'aaa.status' => 'disabled',
	'bridge.1.devname' => 'br0',
	'bridge.1.fd' => '1',
	'bridge.1.port.1.devname' => 'eth0',
	'bridge.1.port.1.status' => 'enabled',
	'bridge.1.port.2.devname' => 'ath0',
	'bridge.1.port.2.status' => 'enabled',
	'bridge.1.stp.status' => 'disabled',
	'bridge.status' => 'disabled',
	'dhcpc.1.devname' => 'br0',
	'dhcpc.1.status' => 'disabled',
	'dhcpc.status' => 'disabled',
	'dhcpd.1.devname' => 'eth0',
	'dhcpd.1.dnsproxy' => 'enabled',
	'dhcpd.1.start' => '10.100.2.10',
	'dhcpd.1.end' => '10.100.2.250',
	'dhcpd.1.lease_time' => '3600',
	'dhcpd.1.netmask' => '255.255.255.0',
	'dhcpd.1.status' => 'enabled',
	'dhcpd.status' => 'enabled',
	'dnsmasq.1.devname' => 'eth0',
	'dnsmasq.1.status' => 'disabled',
	'dnsmasq.status' => 'disabled',
	'ebtables.1.cmd' => '-t nat -A PREROUTING --in-interface ath0 -j arpnat --arpnat-target ACCEPT',
	'ebtables.1.status' => 'enabled',
	'ebtables.2.cmd' => '-t nat -A POSTROUTING --out-interface ath0 -j arpnat --arpnat-target ACCEPT',
	'ebtables.2.status' => 'enabled',
	'ebtables.3.cmd' => '-t broute -A BROUTING --protocol 0x888e --in-interface ath0 -j DROP',
	'ebtables.3.status' => 'enabled',
	'ebtables.50.status' => 'disabled',
	'ebtables.51.status' => 'disabled',
	'ebtables.52.status' => 'disabled',
	'ebtables.status' => 'disabled',
	'httpd.https.port' => '443',
	'httpd.https.status' => 'enabled',
	'httpd.port' => '80',
	'httpd.status' => 'enabled',
	'igmpproxy.status' => 'disabled',
	'iptables.1.status' => 'disabled',
	'iptables.2.status' => 'disabled',
	'iptables.200.status' => 'disabled',
	'iptables.3.status' => 'disabled',
	'iptables.4.status' => 'disabled',
	'iptables.5.status' => 'disabled',
	'iptables.50.status' => 'disabled',
	'iptables.51.status' => 'disabled',
	'iptables.52.status' => 'disabled',
	'iptables.status' => 'disabled',
	'netconf.1.alias.1.status' => 'disabled',
	'netconf.1.alias.2.status' => 'disabled',
	'netconf.1.alias.3.status' => 'disabled',
	'netconf.1.alias.4.status' => 'disabled',
	'netconf.1.alias.5.status' => 'disabled',
	'netconf.1.alias.6.status' => 'disabled',
	'netconf.1.alias.7.status' => 'disabled',
	'netconf.1.alias.8.status' => 'disabled',
	'netconf.1.autoip.status' => 'enabled',
	'netconf.1.devname' => 'eth0',
	'netconf.1.ip' => '10.100.2.1',
	'netconf.1.netmask' => '255.255.255.0',
	'netconf.1.promisc' => 'enabled',
	'netconf.1.status' => 'enabled',
	'netconf.1.up' => 'enabled',
	'netconf.2.alias.1.status' => 'disabled',
	'netconf.2.alias.2.status' => 'disabled',
	'netconf.2.alias.3.status' => 'disabled',
	'netconf.2.alias.4.status' => 'disabled',
	'netconf.2.alias.5.status' => 'disabled',
	'netconf.2.alias.6.status' => 'disabled',
	'netconf.2.alias.7.status' => 'disabled',
	'netconf.2.alias.8.status' => 'disabled',
	'netconf.2.allmulti' => 'disabled',
	'netconf.2.autoip.status' => 'enabled',
	'netconf.2.devname' => 'ath0',
	'netconf.2.ip' => '10.100.0.2',
	'netconf.2.netmask' => '255.255.255.0',
	'netconf.2.status' => 'enabled',
	'netconf.2.up' => 'enabled',
	'netconf.3.alias.1.status' => 'disabled',
	'netconf.3.alias.2.status' => 'disabled',
	'netconf.3.alias.3.status' => 'disabled',
	'netconf.3.alias.4.status' => 'disabled',
	'netconf.3.alias.5.status' => 'disabled',
	'netconf.3.alias.6.status' => 'disabled',
	'netconf.3.alias.7.status' => 'disabled',
	'netconf.3.alias.8.status' => 'disabled',
	'netconf.3.autoip.status' => 'enabled',
	'netconf.3.devname' => 'br0',
	'netconf.3.ip' => '192.168.1.20',
	'netconf.3.netmask' => '255.255.255.0',
	'netconf.3.status' => 'disabled',
	'netconf.3.up' => 'enabled',
	'netconf.status' => 'enabled',
	'netmode' => 'router',
	'ntpclient.1.server' => 'dk.pool.ntp.org',
	'ntpclient.1.status' => 'enabled',
	'ntpclient.status' => 'enabled',
	'ppp.1.password' => '',
	'ppp.1.status' => 'disabled',
	'ppp.status' => 'disabled',
	'pwdog.delay' => '300',
	'pwdog.host' => '10.100.0.1',
	'pwdog.period' => '10',
	'pwdog.retry' => '13',
	'pwdog.status' => 'enabled',
	'radio.1.ack.auto' => 'enabled',
	'radio.1.acktimeout' => '25',
	'radio.1.chanshift' => '0',
	'radio.1.clksel' => '0',
	'radio.1.countrycode' => '1001',
	'radio.1.devname' => 'ath0',
	'radio.1.dfs.status' => '',
	'radio.1.ext_antenna' => 'disabled',
	'radio.1.frag' => 'off',
	'radio.1.ieee_mode' => 'a',
	'radio.1.mcastrate' => '6M',
	'radio.1.mode' => 'managed',
	'radio.1.rate.auto' => 'enabled',
	'radio.1.rate.max' => '54M',
	'radio.1.rts' => 'off',
	'radio.1.rx_antenna' => '1',
	'radio.1.rx_antenna_diversity' => 'disabled',
	'radio.1.status' => 'enabled',
	'radio.1.thresh62a' => '28',
	'radio.1.thresh62b' => '28',
	'radio.1.thresh62g' => '28',
	'radio.1.tx_antenna' => '1',
	'radio.1.tx_antenna_diversity' => 'disabled',
	'radio.1.txpower' => '19',
	'radio.countrycode' => '1001',
	'radio.ratemodule' => 'ath_rate_minstrel',
	'radio.status' => 'enabled',
	'resolv.host.1.name' => 'test-sta',
	'resolv.host.1.status' => 'enabled',
	'resolv.nameserver.1.ip' => '208.67.222.222',
	'resolv.nameserver.1.status' => 'enabled',
	'resolv.nameserver.2.status' => 'enabled',
	'resolv.status' => 'enabled',
	'route.1.devname' => 'br0',
	'route.1.gateway' => '10.100.0.1',
	'route.1.ip' => '0.0.0.0',
	'route.1.netmask' => '0',
	'route.1.status' => 'enabled',
	'route.status' => 'enabled',
	'snmp.community' => 'public',
	'snmp.contact' => 'NrVissing.Net',
	'snmp.location' => 'NrVissing',
	'snmp.status' => 'enabled',
	'sshd.port' => '22',
	'sshd.status' => 'enabled',
	'syslog.remote.ip' => '10.110.0.1',
	'syslog.remote.port' => '514',
	'syslog.remote.status' => 'enabled',
	'syslog.status' => 'enabled',

	'tshaper.in.1.devname'=>'eth0',
	'tshaper.in.burst'=>0,
	'tshaper.in.cburst'=>0,
	'tshaper.in.rate'=>0,
	'tshaper.out.1.devname'=>'ath0',
	'tshaper.out.burst'=>0,
	'tshaper.out.cburst'=>0,
	'tshaper.out.rate'=>2100,
	'tshaper.status'=>'enabled',

	'users.1.name' => 'ubnt',
	'users.1.homedir'=>'/etc/persistent',
	'users.1.status' => 'enabled',
	'users.2.gid' => '100',
	'users.2.name' => 'admin',
	'users.2.shell' => '/bin/false',
	'users.2.status' => 'enabled',
	'users.2.uid' => '100',
	'users.status' => 'enabled',
	'wireless.1.addmtikie' => 'enabled',
	'wireless.1.ap' => '',
	'wireless.1.authmode' => '1',
	'wireless.1.compression' => 'enabled',
	'wireless.1.devname' => 'ath0',
	'wireless.1.fastframes' => 'enabled',
	'wireless.1.frameburst' => 'enabled',
	'wireless.1.hide_ssid' => 'disabled',
	'wireless.1.l2_isolation' => 'disabled',
	'wireless.1.macclone' => 'disabled',
	'wireless.1.scan_list.channels' => '173',
	'wireless.1.scan_list.status' => 'enabled',
	'wireless.1.security' => 'none',
	'wireless.1.signal_led1' => '80',
	'wireless.1.signal_led2' => '60',
	'wireless.1.signal_led3' => '55',
	'wireless.1.signal_led4' => '45',
	'wireless.1.sper' => 'disabled',
	'wireless.1.ssid' => 'UBNT',
	'wireless.1.status' => 'enabled',
	'wireless.1.wds' => 'disabled',
	'wireless.1.wmm' => 'disabled',
	'wireless.1.wmmlevel' => '-1',
	'wireless.status' => 'enabled',
	'wpasupplicant.device.1.devname' => 'ath0',
	'wpasupplicant.device.1.driver' => 'madwifi',
	'wpasupplicant.device.1.profile' => 'WPA-PSK',
	'wpasupplicant.device.1.status' => 'enabled',
	'wpasupplicant.profile.1.name' => 'WPA-PSK',
	'wpasupplicant.profile.1.network.1.bssid' => '',
	'wpasupplicant.profile.1.network.1.key_mgmt.1.name' => 'WPA-PSK',
	'wpasupplicant.profile.1.network.1.pairwise.1.name' => 'CCMP',
	'wpasupplicant.profile.1.network.1.proto.1.name' => 'RSN',
	'wpasupplicant.profile.1.network.1.ssid' => 'UBNT',
	'wpasupplicant.status' => 'enabled',
    },

    ap=>{
	'aaa.1.br.devname' => 'br0',
	'aaa.1.devname' => 'ath0',
	'aaa.1.driver' => 'madwifi',
	'aaa.1.status' => 'enabled',
	'aaa.1.wpa.1.pairwise' => 'CCMP',
	'aaa.1.wpa.key.1.mgmt' => 'WPA-PSK',
	'aaa.1.wpa' => '2',
	'aaa.status' => 'enabled',
	'bridge.1.devname' => 'br0',
	'bridge.1.fd' => '1',
	'bridge.1.port.1.devname' => 'eth0',
	'bridge.1.port.1.status' => 'enabled',
	'bridge.1.port.2.devname' => 'ath0',
	'bridge.1.port.2.status' => 'enabled',
	'bridge.1.stp.status' => 'disabled',
	'bridge.status' => 'disabled',
	'dhcpc.1.devname' => 'br0',
	'dhcpc.1.status' => 'disabled',
	'dhcpc.status' => 'disabled',
	'dhcpd.1.status' => 'disabled',
	'dhcpd.status' => 'disabled',
	'dnsmasq.1.devname' => 'ath0',
	'dnsmasq.1.status' => 'enabled',
	'dnsmasq.status' => 'enabled',
	'ebtables.1.cmd' => '-t nat -A PREROUTING --in-interface ath0 -j arpnat --arpnat-target ACCEPT',
	'ebtables.1.status' => 'disabled',
	'ebtables.2.cmd' => '-t nat -A POSTROUTING --out-interface ath0 -j arpnat --arpnat-target ACCEPT',
	'ebtables.2.status' => 'disabled',
	'ebtables.3.cmd' => '-t broute -A BROUTING --protocol 0x888e --in-interface ath0 -j DROP',
	'ebtables.3.status' => 'enabled',
	'ebtables.50.status' => 'disabled',
	'ebtables.51.status' => 'disabled',
	'ebtables.52.status' => 'disabled',
	'ebtables.status' => 'disabled',
	'httpd.https.port' => '443',
	'httpd.https.status' => 'enabled',
	'httpd.port' => '80',
	'httpd.status' => 'enabled',
	'igmpproxy.status' => 'disabled',
	'iptables.1.status' => 'disabled',
	'iptables.2.status' => 'disabled',
	'iptables.200.status' => 'disabled',
	'iptables.3.status' => 'disabled',
	'iptables.4.status' => 'disabled',
	'iptables.5.status' => 'disabled',
	'iptables.50.status' => 'disabled',
	'iptables.51.status' => 'disabled',
	'iptables.52.status' => 'disabled',
	'iptables.status' => 'disabled',
	'netconf.1.alias.1.status' => 'disabled',
	'netconf.1.alias.2.status' => 'disabled',
	'netconf.1.alias.3.status' => 'disabled',
	'netconf.1.alias.4.status' => 'disabled',
	'netconf.1.alias.5.status' => 'disabled',
	'netconf.1.alias.6.status' => 'disabled',
	'netconf.1.alias.7.status' => 'disabled',
	'netconf.1.alias.8.status' => 'disabled',
	'netconf.1.autoip.status' => 'enabled',
	'netconf.1.devname' => 'eth0',
	'netconf.1.netmask' => '255.255.255.0',
	'netconf.1.promisc' => 'enabled',
	'netconf.1.status' => 'enabled',
	'netconf.1.up' => 'enabled',
	'netconf.2.alias.1.status' => 'disabled',
	'netconf.2.alias.2.status' => 'disabled',
	'netconf.2.alias.3.status' => 'disabled',
	'netconf.2.alias.4.status' => 'disabled',
	'netconf.2.alias.5.status' => 'disabled',
	'netconf.2.alias.6.status' => 'disabled',
	'netconf.2.alias.7.status' => 'disabled',
	'netconf.2.alias.8.status' => 'disabled',
	'netconf.2.allmulti' => 'disabled',
	'netconf.2.autoip.status' => 'enabled',
	'netconf.2.devname' => 'ath0',
	'netconf.2.netmask' => '255.255.255.0',
	'netconf.2.status' => 'enabled',
	'netconf.2.up' => 'enabled',
	'netconf.3.alias.1.status' => 'disabled',
	'netconf.3.alias.2.status' => 'disabled',
	'netconf.3.alias.3.status' => 'disabled',
	'netconf.3.alias.4.status' => 'disabled',
	'netconf.3.alias.5.status' => 'disabled',
	'netconf.3.alias.6.status' => 'disabled',
	'netconf.3.alias.7.status' => 'disabled',
	'netconf.3.alias.8.status' => 'disabled',
	'netconf.3.autoip.status' => 'enabled',
	'netconf.3.devname' => 'br0',
	'netconf.3.ip' => '192.168.1.20',
	'netconf.3.netmask' => '255.255.255.0',
	'netconf.3.status' => 'disabled',
	'netconf.3.up' => 'enabled',
	'netconf.status' => 'enabled',
	'netmode' => 'router',
	'ntpclient.1.server' => 'dk.pool.ntp.org',
	'ntpclient.1.status' => 'enabled',
	'ntpclient.status' => 'enabled',
	'ppp.1.password' => '',
	'ppp.1.status' => 'disabled',
	'ppp.status' => 'disabled',
	'radio.1.ack.auto' => 'disabled',
	'radio.1.chanshift' => '0',
	'radio.1.clksel' => '0',
	'radio.1.countrycode' => '1001',
	'radio.1.devname' => 'ath0',
	'radio.1.dfs.status' => '',
	'radio.1.ext_antenna' => 'disabled',
	'radio.1.frag' => 'off',
	'radio.1.ieee_mode' => 'a',
	'radio.1.mcastrate' => '6M',
	'radio.1.mode' => 'master',
	'radio.1.rate.auto' => 'enabled',
	'radio.1.rate.max' => '54M',
	'radio.1.rts' => 'off',
	'radio.1.rx_antenna_diversity' => 'disabled',
	'radio.1.status' => 'enabled',
	'radio.1.thresh62a' => '28',
	'radio.1.thresh62b' => '28',
	'radio.1.thresh62g' => '28',
	'radio.1.tx_antenna_diversity' => 'disabled',
	'radio.1.txpower' => '19',
	'radio.countrycode' => '1001',
	'radio.ratemodule' => 'ath_rate_minstrel',
	'radio.status' => 'enabled',
	'resolv.host.1.status' => 'enabled',
	'resolv.nameserver.1.status' => 'enabled',
	'resolv.nameserver.2.status' => 'enabled',
	'resolv.status' => 'enabled',
	'route.1.devname' => 'eth0',
	'route.1.ip' => '0.0.0.0',
	'route.1.netmask' => '0',
	'route.1.status' => 'enabled',
	'route.status' => 'enabled',
	'snmp.community' => 'public',
	'snmp.contact' => 'NrVissing.Net',
	'snmp.status' => 'enabled',
	'sshd.port' => '22',
	'sshd.status' => 'enabled',
	'syslog.remote.ip' => '10.110.0.1',
	'syslog.remote.port' => '514',
	'syslog.remote.status' => 'enabled',
	'syslog.status' => 'enabled',
	'tshaper.status' => 'disabled',
	'users.1.name' => 'ubnt',
	'users.1.status' => 'enabled',
	'users.1.homedir'=>'/etc/persistent',
	'users.2.gid' => '100',
	'users.2.name' => 'admin',
	'users.2.shell' => '/bin/false',
	'users.2.status' => 'enabled',
	'users.2.uid' => '100',
	'users.status' => 'enabled',
	'wireless.1.addmtikie' => 'enabled',
	'wireless.1.ap' => '',
	'wireless.1.authmode' => '1',
	'wireless.1.compression' => 'enabled',
	'wireless.1.devname' => 'ath0',
	'wireless.1.fastframes' => 'enabled',
	'wireless.1.frameburst' => 'enabled',
	'wireless.1.hide_ssid' => 'disabled',
	'wireless.1.l2_isolation' => 'disabled',
	'wireless.1.mac_acl.1.mac' => '',
	'wireless.1.mac_acl.1.status' => 'disabled',
	'wireless.1.mac_acl.10.mac' => '',
	'wireless.1.mac_acl.10.status' => 'disabled',
	'wireless.1.mac_acl.11.mac' => '',
	'wireless.1.mac_acl.11.status' => 'disabled',
	'wireless.1.mac_acl.12.mac' => '',
	'wireless.1.mac_acl.12.status' => 'disabled',
	'wireless.1.mac_acl.13.mac' => '',
	'wireless.1.mac_acl.13.status' => 'disabled',
	'wireless.1.mac_acl.14.mac' => '',
	'wireless.1.mac_acl.14.status' => 'disabled',
	'wireless.1.mac_acl.15.mac' => '',
	'wireless.1.mac_acl.15.status' => 'disabled',
	'wireless.1.mac_acl.16.mac' => '',
	'wireless.1.mac_acl.16.status' => 'disabled',
	'wireless.1.mac_acl.2.mac' => '',
	'wireless.1.mac_acl.2.status' => 'disabled',
	'wireless.1.mac_acl.3.mac' => '',
	'wireless.1.mac_acl.3.status' => 'disabled',
	'wireless.1.mac_acl.4.mac' => '',
	'wireless.1.mac_acl.4.status' => 'disabled',
	'wireless.1.mac_acl.5.mac' => '',
	'wireless.1.mac_acl.5.status' => 'disabled',
	'wireless.1.mac_acl.6.mac' => '',
	'wireless.1.mac_acl.6.status' => 'disabled',
	'wireless.1.mac_acl.7.mac' => '',
	'wireless.1.mac_acl.7.status' => 'disabled',
	'wireless.1.mac_acl.8.mac' => '',
	'wireless.1.mac_acl.8.status' => 'disabled',
	'wireless.1.mac_acl.9.mac' => '',
	'wireless.1.mac_acl.9.status' => 'disabled',
	'wireless.1.mac_acl.policy' => 'allow',
	'wireless.1.mac_acl.status' => 'disabled',
	'wireless.1.macclone' => 'disabled',
	'wireless.1.security' => 'none',
	'wireless.1.signal_led1' => '80',
	'wireless.1.signal_led2' => '60',
	'wireless.1.signal_led3' => '55',
	'wireless.1.signal_led4' => '45',
	'wireless.1.sper' => 'disabled',
	'wireless.1.status' => 'enabled',
	'wireless.1.wds' => 'disabled',
	'wireless.1.wmm' => 'disabled',
	'wireless.1.wmmlevel' => '-1',
	'wireless.status' => 'enabled',
	'wpasupplicant.device.1.profile' => '',
	'wpasupplicant.device.1.status' => 'disabled',
	'wpasupplicant.profile.1.name' => '',
	'wpasupplicant.profile.1.network.1.bssid' => '',
	'wpasupplicant.profile.1.network.1.key_mgmt.1.name' => '',
	'wpasupplicant.status' => 'disabled',
    },
    
    'm-ap' => {
          'aaa.1.br.devname' => 'br0',
          'aaa.1.devname' => 'ath0',
          'aaa.1.driver' => 'madwifi',
          'aaa.1.ssid' => 'NrVissing.Net-300m',
          'aaa.1.status' => 'enabled',
          'aaa.1.wpa' => '2',
          'aaa.1.wpa.1.pairwise' => 'CCMP',
          'aaa.1.wpa.key.1.mgmt' => 'WPA-PSK',
          'aaa.1.wpa.psk' => 'testfest',
          'aaa.status' => 'enabled',
          'bridge.1.devname' => 'br0',
          'bridge.1.fd' => '1',
          'bridge.1.port.1.devname' => 'eth0',
          'bridge.1.port.1.prio' => '10',
          'bridge.1.port.1.status' => 'enabled',
          'bridge.1.port.2.devname' => 'ath0',
          'bridge.1.port.2.prio' => '30',
          'bridge.1.port.2.status' => 'enabled',
          'bridge.1.port.3.devname' => 'eth1',
          'bridge.1.port.3.prio' => '20',
          'bridge.1.port.3.status' => 'enabled',
          'bridge.1.stp.status' => 'disabled',
          'bridge.status' => 'disabled',
          'dhcpc.1.devname' => 'eth0',
          'dhcpc.1.fallback' => '192.168.1.20',
          'dhcpc.1.status' => 'disabled',
          'dhcpc.status' => 'disabled',
          'dhcpd.1.devname' => 'ath0',
          'dhcpd.1.dnsproxy' => 'enabled',
          'dhcpd.1.end' => '10.0.27.200',
          'dhcpd.1.lease_time' => '3600',
          'dhcpd.1.netmask' => '255.255.255.0',
          'dhcpd.1.start' => '10.0.27.20',
          'dhcpd.1.status' => 'enabled',
          'dhcpd.status' => 'disabled',
          'dnsmasq.1.devname' => 'ath0',
          'dnsmasq.1.status' => 'disabled',
          'dnsmasq.status' => 'disabled',
          'ebtables.1.cmd' => '-t nat -A PREROUTING --in-interface ath0 -j arpnat --arpnat-target ACCEPT',
          'ebtables.1.status' => 'enabled',
          'ebtables.2.cmd' => '-t nat -A POSTROUTING --out-interface ath0 -j arpnat --arpnat-target ACCEPT',
          'ebtables.2.status' => 'enabled',
          'ebtables.3.cmd' => '-t broute -A BROUTING --protocol 0x888e --in-interface ath0 -j DROP',
          'ebtables.3.status' => 'enabled',
          'ebtables.50.status' => 'disabled',
          'ebtables.51.status' => 'disabled',
          'ebtables.52.status' => 'disabled',
          'ebtables.status' => 'disabled',
          'gpio.1.direction' => '1',
          'gpio.1.line' => '8',
          'gpio.1.status' => 'enabled',
          'gpio.1.value' => 'disabled',
          'gpio.status' => 'enabled',
          'gui.language' => 'en_US',
          'httpd.https.port' => '443',
          'httpd.https.status' => 'enabled',
          'httpd.port' => '80',
          'httpd.port.http' => '80',
          'httpd.session.timeout' => '900',
          'httpd.status' => 'enabled',
          'igmpproxy.status' => 'disabled',
          'iptables.1.status' => 'disabled',
          'iptables.2.status' => 'disabled',
          'iptables.200.status' => 'disabled',
          'iptables.3.status' => 'disabled',
          'iptables.4.status' => 'disabled',
          'iptables.5.status' => 'disabled',
          'iptables.50.status' => 'disabled',
          'iptables.51.status' => 'disabled',
          'iptables.52.status' => 'disabled',
          'iptables.status' => 'disabled',
          'netconf.1.alias.1.status' => 'disabled',
          'netconf.1.alias.2.status' => 'disabled',
          'netconf.1.alias.3.status' => 'disabled',
          'netconf.1.alias.4.status' => 'disabled',
          'netconf.1.alias.5.status' => 'disabled',
          'netconf.1.alias.6.status' => 'disabled',
          'netconf.1.alias.7.status' => 'disabled',
          'netconf.1.alias.8.status' => 'disabled',
          'netconf.1.autoip.status' => 'enabled',
          'netconf.1.autoneg' => 'enabled',
          'netconf.1.devname' => 'eth0',
          'netconf.1.duplex' => 'enabled',
          'netconf.1.ip' => '192.168.1.30',
          'netconf.1.netmask' => '255.255.255.0',
          'netconf.1.promisc' => 'enabled',
          'netconf.1.speed' => '100',
          'netconf.1.status' => 'enabled',
          'netconf.1.up' => 'enabled',
          'netconf.2.alias.1.status' => 'disabled',
          'netconf.2.alias.2.status' => 'disabled',
          'netconf.2.alias.3.status' => 'disabled',
          'netconf.2.alias.4.status' => 'disabled',
          'netconf.2.alias.5.status' => 'disabled',
          'netconf.2.alias.6.status' => 'disabled',
          'netconf.2.alias.7.status' => 'disabled',
          'netconf.2.alias.8.status' => 'disabled',
          'netconf.2.allmulti' => 'enabled',
          'netconf.2.autoip.status' => 'enabled',
          'netconf.2.devname' => 'ath0',
          'netconf.2.ip' => '10.0.27.254',
          'netconf.2.netmask' => '255.255.255.0',
          'netconf.2.promisc' => 'enabled',
          'netconf.2.status' => 'enabled',
          'netconf.2.up' => 'enabled',
          'netconf.3.alias.1.status' => 'disabled',
          'netconf.3.alias.2.status' => 'disabled',
          'netconf.3.alias.3.status' => 'disabled',
          'netconf.3.alias.4.status' => 'disabled',
          'netconf.3.alias.5.status' => 'disabled',
          'netconf.3.alias.6.status' => 'disabled',
          'netconf.3.alias.7.status' => 'disabled',
          'netconf.3.alias.8.status' => 'disabled',
          'netconf.3.autoip.status' => 'enabled',
          'netconf.3.devname' => 'br0',
          'netconf.3.ip' => '192.168.1.20',
          'netconf.3.netmask' => '255.255.255.0',
          'netconf.3.status' => 'disabled',
          'netconf.3.up' => 'enabled',
          'netconf.status' => 'enabled',
          'netmode' => 'router',
          'ntpclient.1.server' => 'dk.pool.ntp.org',
          'ntpclient.1.status' => 'enabled',
          'ntpclient.status' => 'enabled',
          'ppp.1.password' => '',
          'ppp.1.status' => 'disabled',
          'ppp.status' => 'disabled',
          'pwdog.delay' => '300',
          'pwdog.host' => '127.0.0.1',
          'pwdog.period' => '10',
          'pwdog.retry' => '6',
          'pwdog.status' => 'enabled',
          'radio.1.ack.auto' => 'enabled',
          'radio.1.ackdistance' => '300',
          'radio.1.acktimeout' => '23',
          'radio.1.ampdu.bytes' => '50000',
          'radio.1.ampdu.frames' => '32',
          'radio.1.ampdu.status' => 'enabled',
          'radio.1.chanshift' => '0',
          'radio.1.clksel' => '1',
          'radio.1.countrycode' => '511',
          'radio.1.cwm.enable' => '0',
          'radio.1.cwm.mode' => '0',
          'radio.1.devname' => 'ath0',
          'radio.1.dfs.status' => '',
          'radio.1.forbiasauto' => '0',
          'radio.1.frag' => 'off',
          'radio.1.freq' => '5500',
          'radio.1.ieee_mode' => '11naht20',
          'radio.1.mcastrate' => '',
          'radio.1.mode' => 'master',
          'radio.1.obey' => 'disabled',
          'radio.1.polling' => 'enabled',
          'radio.1.pollingnoack' => '',
          'radio.1.pollingpri' => '',
          'radio.1.rate.auto' => 'enabled',
          'radio.1.rate.mcs' => '15',
          'radio.1.rts' => 'off',
          'radio.1.status' => 'enabled',
          'radio.1.subsystemid' => '0xe005',
          'radio.1.thresh62a' => '',
          'radio.1.thresh62b' => '',
          'radio.1.thresh62g' => '',
          'radio.1.txpower' => '23',
          'radio.countrycode' => '511',
          'radio.status' => 'enabled',
          'resolv.host.1.name' => 'UBNT',
          'resolv.host.1.status' => 'enabled',
          'resolv.nameserver.1.ip' => '10.110.0.1',
          'resolv.nameserver.1.status' => 'enabled',
          'resolv.nameserver.2.ip' => '10.200.0.1',
          'resolv.nameserver.2.status' => 'enabled',
          'resolv.status' => 'enabled',
          'route.1.devname' => 'eth0',
          'route.1.gateway' => '192.168.1.1',
          'route.1.ip' => '0.0.0.0',
          'route.1.netmask' => '0',
          'route.1.status' => 'enabled',
          'route.status' => 'enabled',
	  'snmp.community' => 'public',
	  'snmp.contact' => 'NrVissing.Net',
	  'snmp.status' => 'enabled',
          'sshd.auth.passwd' => 'enabled',
          'sshd.port' => '22',
          'sshd.status' => 'enabled',
          'syslog.remote.ip' => '10.110.0.1',
          'syslog.remote.port' => '514',
          'syslog.remote.status' => 'enabled',
          'syslog.status' => 'enabled',
          'telnetd.status' => 'disabled',
          'tshaper.status' => 'disabled',
          'users.1.name' => 'ubnt',
          'users.1.password' => 'VvpvCwhccFv6Q',
          'users.1.status' => 'enabled',
          'users.2.gid' => '100',
          'users.2.name' => 'admin',
          'users.2.password' => 'Ee4Pglz1y/jKw',
          'users.2.shell' => '/bin/false',
          'users.2.status' => 'enabled',
          'users.2.uid' => '100',
          'users.status' => 'enabled',
          'wireless.1.addmtikie' => 'enabled',
          'wireless.1.ap' => '',
          'wireless.1.authmode' => '1',
          'wireless.1.compression' => '0',
          'wireless.1.devname' => 'ath0',
          'wireless.1.fastframes' => '0',
          'wireless.1.frameburst' => '0',
          'wireless.1.hide_ssid' => 'disabled',
          'wireless.1.l2_isolation' => 'disabled',
          'wireless.1.mac_acl.1.mac' => '',
          'wireless.1.mac_acl.1.status' => 'disabled',
          'wireless.1.mac_acl.10.mac' => '',
          'wireless.1.mac_acl.10.status' => 'disabled',
          'wireless.1.mac_acl.11.mac' => '',
          'wireless.1.mac_acl.11.status' => 'disabled',
          'wireless.1.mac_acl.12.mac' => '',
          'wireless.1.mac_acl.12.status' => 'disabled',
          'wireless.1.mac_acl.13.mac' => '',
          'wireless.1.mac_acl.13.status' => 'disabled',
          'wireless.1.mac_acl.14.mac' => '',
          'wireless.1.mac_acl.14.status' => 'disabled',
          'wireless.1.mac_acl.15.mac' => '',
          'wireless.1.mac_acl.15.status' => 'disabled',
          'wireless.1.mac_acl.16.mac' => '',
          'wireless.1.mac_acl.16.status' => 'disabled',
          'wireless.1.mac_acl.2.mac' => '',
          'wireless.1.mac_acl.2.status' => 'disabled',
          'wireless.1.mac_acl.3.mac' => '',
          'wireless.1.mac_acl.3.status' => 'disabled',
          'wireless.1.mac_acl.4.mac' => '',
          'wireless.1.mac_acl.4.status' => 'disabled',
          'wireless.1.mac_acl.5.mac' => '',
          'wireless.1.mac_acl.5.status' => 'disabled',
          'wireless.1.mac_acl.6.mac' => '',
          'wireless.1.mac_acl.6.status' => 'disabled',
          'wireless.1.mac_acl.7.mac' => '',
          'wireless.1.mac_acl.7.status' => 'disabled',
          'wireless.1.mac_acl.8.mac' => '',
          'wireless.1.mac_acl.8.status' => 'disabled',
          'wireless.1.mac_acl.9.mac' => '',
          'wireless.1.mac_acl.9.status' => 'disabled',
          'wireless.1.mac_acl.policy' => 'allow',
          'wireless.1.mac_acl.status' => 'disabled',
          'wireless.1.macclone' => 'disabled',
          'wireless.1.scan_list.status' => 'disabled',
          'wireless.1.security' => 'none',
          'wireless.1.signal_led1' => '80',
          'wireless.1.signal_led2' => '65',
          'wireless.1.signal_led3' => '60',
          'wireless.1.signal_led4' => '55',
          'wireless.1.ssid' => 'NrVissing.Net-300m',
          'wireless.1.status' => 'enabled',
          'wireless.1.wds' => 'disabled',
          'wireless.1.wmm' => 'enabled',
          'wireless.1.wmmlevel' => '',
          'wireless.status' => 'enabled',
          'wpasupplicant.device.1.profile' => '',
          'wpasupplicant.device.1.status' => 'disabled',
          'wpasupplicant.profile.1.name' => '',
          'wpasupplicant.profile.1.network.1.bssid' => '',
          'wpasupplicant.profile.1.network.1.eap.1.status' => 'disabled',
          'wpasupplicant.profile.1.network.1.key_mgmt.1.name' => '',
          'wpasupplicant.status' => 'disabled',	
    },

    'm-sta'=>{
          'aaa.1.status' => 'disabled',
          'aaa.1.wpa.key.1.mgmt' => 'WPA-PSK',
          'aaa.status' => 'disabled',
          'bridge.1.devname' => 'br0',
          'bridge.1.fd' => '1',
          'bridge.1.port.1.devname' => 'eth0',
          'bridge.1.port.1.prio' => '10',
          'bridge.1.port.1.status' => 'enabled',
          'bridge.1.port.2.devname' => 'ath0',
          'bridge.1.port.2.prio' => '30',
          'bridge.1.port.2.status' => 'enabled',
          'bridge.1.port.3.devname' => 'eth1',
          'bridge.1.port.3.prio' => '20',
          'bridge.1.port.3.status' => 'enabled',
          'bridge.1.stp.status' => 'disabled',
          'bridge.status' => 'disabled',
          'dhcpc.1.devname' => 'br0',
          'dhcpc.1.status' => 'disabled',
          'dhcpc.status' => 'disabled',

          'dhcpd.1.devname' => 'eth0',
          'dhcpd.1.dnsproxy' => 'enabled',
          #'dhcpd.1.end' => '10.103.47.147',
          'dhcpd.1.lease_time' => '3600',
          'dhcpd.1.netmask' => '255.255.255.0',
          #'dhcpd.1.start' => '10.103.47.47',
          'dhcpd.1.status' => 'enabled',
          'dhcpd.status' => 'enabled',

          'dnsmasq.1.devname' => 'eth0',
          'dnsmasq.1.status' => 'enabled',
          'dnsmasq.status' => 'enabled',
          'ebtables.1.cmd' => '-t nat -A PREROUTING --in-interface ath0 -j arpnat --arpnat-target ACCEPT',
          'ebtables.1.status' => 'enabled',
          'ebtables.2.cmd' => '-t nat -A POSTROUTING --out-interface ath0 -j arpnat --arpnat-target ACCEPT',
          'ebtables.2.status' => 'enabled',
          'ebtables.3.cmd' => '-t broute -A BROUTING --protocol 0x888e --in-interface ath0 -j DROP',
          'ebtables.3.status' => 'enabled',
          'ebtables.50.status' => 'disabled',
          'ebtables.51.status' => 'disabled',
          'ebtables.52.status' => 'disabled',
          'ebtables.status' => 'disabled',
          'gpio.1.direction' => '1',
          'gpio.1.line' => '8',
          'gpio.1.status' => 'enabled',
          'gpio.1.value' => 'disabled',
          'gpio.status' => 'enabled',
          'gui.language' => 'en_US',
          'httpd.https.port' => '443',
          'httpd.https.status' => 'enabled',
          'httpd.port' => '80',
          'httpd.port.http' => '80',
          'httpd.session.timeout' => '7200',
          'httpd.status' => 'enabled',
          'igmpproxy.status' => 'disabled',
          'iptables.1.status' => 'disabled',
          'iptables.2.status' => 'disabled',
          'iptables.200.status' => 'disabled',
          'iptables.3.status' => 'disabled',
          'iptables.4.status' => 'disabled',
          'iptables.5.status' => 'disabled',
          'iptables.50.status' => 'disabled',
          'iptables.51.status' => 'disabled',
          'iptables.52.status' => 'disabled',
          'iptables.status' => 'disabled',
          'netconf.1.alias.1.status' => 'disabled',
          'netconf.1.alias.2.status' => 'disabled',
          'netconf.1.alias.3.status' => 'disabled',
          'netconf.1.alias.4.status' => 'disabled',
          'netconf.1.alias.5.status' => 'disabled',
          'netconf.1.alias.6.status' => 'disabled',
          'netconf.1.alias.7.status' => 'disabled',
          'netconf.1.alias.8.status' => 'disabled',
          'netconf.1.autoip.status' => 'enabled',
          'netconf.1.autoneg' => 'enabled',
          'netconf.1.devname' => 'eth0',
          'netconf.1.duplex' => 'enabled',
          'netconf.1.ip' => '192.168.1.20',
          'netconf.1.netmask' => '255.255.255.0',
          'netconf.1.promisc' => 'enabled',
          'netconf.1.speed' => '100',
          'netconf.1.status' => 'enabled',
          'netconf.1.up' => 'enabled',
          'netconf.2.alias.1.status' => 'disabled',
          'netconf.2.alias.2.status' => 'disabled',
          'netconf.2.alias.3.status' => 'disabled',
          'netconf.2.alias.4.status' => 'disabled',
          'netconf.2.alias.5.status' => 'disabled',
          'netconf.2.alias.6.status' => 'disabled',
          'netconf.2.alias.7.status' => 'disabled',
          'netconf.2.alias.8.status' => 'disabled',
          'netconf.2.allmulti' => 'enabled',
          'netconf.2.autoip.status' => 'enabled',
          'netconf.2.devname' => 'ath0',
          'netconf.2.ip' => '192.168.47.1',
          'netconf.2.netmask' => '255.255.255.0',
          'netconf.2.promisc' => 'enabled',
          'netconf.2.status' => 'enabled',
          'netconf.2.up' => 'enabled',
          'netconf.3.alias.1.status' => 'disabled',
          'netconf.3.alias.2.status' => 'disabled',
          'netconf.3.alias.3.status' => 'disabled',
          'netconf.3.alias.4.status' => 'disabled',
          'netconf.3.alias.5.status' => 'disabled',
          'netconf.3.alias.6.status' => 'disabled',
          'netconf.3.alias.7.status' => 'disabled',
          'netconf.3.alias.8.status' => 'disabled',
          'netconf.3.autoip.status' => 'enabled',
          'netconf.3.devname' => 'br0',
          'netconf.3.ip' => '192.168.1.20',
          'netconf.3.netmask' => '255.255.255.0',
          'netconf.3.status' => 'disabled',
          'netconf.3.up' => 'enabled',
          'netconf.status' => 'enabled',
          'netmode' => 'router',
          'ntpclient.1.server' => 'dk.pool.ntp.org',
          'ntpclient.1.status' => 'enabled',
          'ntpclient.status' => 'enabled',
          'ppp.1.password' => '',
          'ppp.1.status' => 'disabled',
          'ppp.status' => 'disabled',
          'pwdog.delay' => '300',
          'pwdog.host' => '127.0.0.1',
          'pwdog.period' => '10',
          'pwdog.retry' => '12',
          'pwdog.status' => 'enabled',
          'radio.1.ack.auto' => 'enabled',
          'radio.1.ackdistance' => '600',
          'radio.1.acktimeout' => '25',
          'radio.1.ampdu.bytes' => '50000',
          'radio.1.ampdu.frames' => '32',
          'radio.1.ampdu.status' => 'enabled',
          'radio.1.chanshift' => '0',
          'radio.1.clksel' => '1',
          'radio.1.countrycode' => '511',
          'radio.1.cwm.enable' => '0',
          'radio.1.cwm.mode' => '1',
          'radio.1.devname' => 'ath0',
          'radio.1.dfs.status' => '',
          'radio.1.forbiasauto' => '0',
          'radio.1.frag' => 'off',
          'radio.1.ieee_mode' => '11naht40',
          'radio.1.mcastrate' => '',
          'radio.1.mode' => 'managed',
          'radio.1.obey' => 'disabled',
          'radio.1.polling' => 'disabled',
          'radio.1.pollingnoack' => '',
          'radio.1.pollingpri' => '3',
          'radio.1.rate.auto' => 'enabled',
          'radio.1.rate.mcs' => '15',
          'radio.1.rts' => 'off',
          'radio.1.status' => 'enabled',
          'radio.1.subsystemid' => '0xe005',
          'radio.1.thresh62a' => '',
          'radio.1.thresh62b' => '',
          'radio.1.thresh62g' => '',
          'radio.1.txpower' => '23',
          'radio.countrycode' => '511',
          'radio.status' => 'enabled',
          'resolv.host.1.name' => 'defaultconfig',
          'resolv.host.1.status' => 'enabled',
          'resolv.nameserver.1.ip' => '10.110.0.1',
          'resolv.nameserver.1.status' => 'enabled',
          'resolv.nameserver.2.ip' => '10.200.0.1',
          'resolv.nameserver.2.status' => 'enabled',
          'resolv.status' => 'enabled',
          'route.1.devname' => 'ath0',
          'route.1.gateway' => '192.168.47.254',
          'route.1.ip' => '0.0.0.0',
          'route.1.netmask' => '0',
          'route.1.status' => 'enabled',
          'route.status' => 'enabled',
          'snmp.community' => 'public',
          'snmp.contact' => 'NrVissing.Net',
          'snmp.location' => 'nrVissing',
          'snmp.status' => 'enabled',
          'sshd.auth.passwd' => 'enabled',
          'sshd.port' => '22',
          'sshd.status' => 'enabled',
          'syslog.remote.ip' => '10.110.0.1',
          'syslog.remote.port' => '514',
          'syslog.remote.status' => 'enabled',
          'syslog.status' => 'enabled',
          'telnetd.status' => 'disabled',
          'tshaper.in.1.devname' => 'eth0',
          'tshaper.in.burst' => '0',
          'tshaper.in.cburst' => '0',
          'tshaper.in.rate' => '0',
          'tshaper.out.1.devname' => 'ath0',
          'tshaper.out.burst' => '0',
          'tshaper.out.cburst' => '0',
          'tshaper.out.rate' => '2000',
          'tshaper.status' => 'enabled',
          'users.1.name' => 'ubnt',
          'users.1.password' => 'VvpvCwhccFv6Q',
          'users.1.status' => 'enabled',
          'users.2.gid' => '100',
          'users.2.name' => 'admin',
          'users.2.password' => 'VvIVBfPsNQg36',
          'users.2.shell' => '/bin/false',
          'users.2.status' => 'enabled',
          'users.2.uid' => '100',
          'users.status' => 'enabled',
          'wireless.1.addmtikie' => 'enabled',
          'wireless.1.ap' => '',
          'wireless.1.authmode' => '1',
          'wireless.1.compression' => '0',
          'wireless.1.devname' => 'ath0',
          'wireless.1.fastframes' => '0',
          'wireless.1.frameburst' => '0',
          'wireless.1.hide_ssid' => 'disabled',
          'wireless.1.l2_isolation' => 'disabled',
          'wireless.1.macclone' => 'disabled',
          'wireless.1.scan_list.channels' => '5500',
          'wireless.1.scan_list.status' => 'enabled',
          'wireless.1.security' => 'none',
          'wireless.1.signal_led1' => '80',
          'wireless.1.signal_led2' => '65',
          'wireless.1.signal_led3' => '60',
          'wireless.1.signal_led4' => '55',
          'wireless.1.ssid' => 'NrVissing.Net-103-6',
          'wireless.1.status' => 'enabled',
          'wireless.1.wds' => 'disabled',
          'wireless.1.wmm' => 'enabled',
          'wireless.1.wmmlevel' => '',
          'wireless.status' => 'enabled',
          'wpasupplicant.device.1.devname' => 'ath0',
          'wpasupplicant.device.1.driver' => 'madwifi',
          'wpasupplicant.device.1.profile' => 'WPA-PSK',
          'wpasupplicant.device.1.status' => 'enabled',
          'wpasupplicant.profile.1.name' => 'WPA-PSK',
          'wpasupplicant.profile.1.network.1.bssid' => '',
          'wpasupplicant.profile.1.network.1.eap.1.status' => 'disabled',
          'wpasupplicant.profile.1.network.1.key_mgmt.1.name' => 'WPA-PSK',
          'wpasupplicant.profile.1.network.1.pairwise.1.name' => 'CCMP',
          'wpasupplicant.profile.1.network.1.proto.1.name' => 'RSN',
          'wpasupplicant.profile.1.network.1.ssid' => 'NrVissing.Net-103-6',
          'wpasupplicant.status' => 'enabled'
    }
);

sub defaultConfig($) {
#    my $name = shift;
#    return %{$CONFIGS{$name}} or die "Invalid config name: $name";
    die "Fail!";
}

my %FREQ = (
    1 => 2412,
    2 => 2417,
    3 => 2422,
    4 => 2427,
    5 => 2432,
    6 => 2437,
    7 => 2442,
    8 => 2447,
    9 => 2452,
    10 => 2457,
    11 => 2462,
    12 => 2467,
    13 => 2472,
    14 => 2484,
    34 => 5170,
    36 => 5180,
    38 => 5190,
    40 => 5200,
    42 => 5210,
    44 => 5220,
    46 => 5230,
    48 => 5240,
    50 => 5250,
    52 => 5260,
    56 => 5280,
    58 => 5290,
    60 => 5300,
    64 => 5320,
    100 => 5500,
    104 => 5520,
    108 => 5540,
    112 => 5560,
    116 => 5580,
    120 => 5600,
    124 => 5620,
    128 => 5640,
    132 => 5660,
    136 => 5680,
    140 => 5700,
    149 => 5745,
    152 => 5760,
    153 => 5765,
    157 => 5785,
    160 => 5800,
    161 => 5805,
    165 => 5825,
    169 => 5845,
    173 => 5865,
);

sub channel2freq($) {
    my ($ch) = @_;

    die "Failed to look up frequency of channel $ch" unless $FREQ{$ch};

    return $FREQ{$ch};
}

package AirOSAgent;
use LWP::UserAgent;
use HTTP::Request::Common;
use URI;
use Archive::Tar;
use Data::Dumper;

our @ISA = ('LWP::UserAgent');

sub new {
    my $classname = shift;

    my $self = LWP::UserAgent::new($classname, keep_alive => 1);
    bless $self, $classname;
    
    my $cj = {
	ui_language=>'en_US',
	av_warning=>'skip',
	last_check=>time-10,
    };

    $self->cookie_jar($cj);
    $self->ssl_opts(verify_hostname=>0);

    return $self;
}


sub redirect_ok {
  my ($self, $request) = @_;
  $request->method("GET"),$request->content("") if $request->method eq "POST";
  1;
}

sub req($$) {
    my ($self, $req) = @_;
    $req->authorization_basic('ubnt', 'ubnt');	

    my $res = $self->request($req);
    print "We're at: ".$res->request->uri."\n";
    if ($res->request->uri =~ /login.cgi/) {

	my $uri = new URI($req->uri);
	$uri->path_query("/login.cgi");

	my $lr = POST "$uri", 
	Content_Type => 'multipart/form-data',
	Content => [
	    uri=>"/login.cgi",
	    username=>'ubnt',
	    password=>'ubnt',	    
	    ];

	my $lres = $self->request($lr);

	die "Unable to log in"
	    unless $lres->is_success;

	my $res2 = $self->request($req);

	return $res2;
    }

    return $res;
}


sub getFirmwareVersionM {
    my ($self, $ip) = @_;

#    my $req = POST "http://$ip/support.cgi";
    my $req = GET "http://$ip/support.cgi";
    my $res = $self->req($req);
    return if $res->code == 404;
    
    die "Unable to get support data: ".$res->status_line.
	Dumper $res
	if $res->is_error;

    my $tarFile = "/tmp/AirOS.pm.$$.tar";
    open T, ">$tarFile" or die "Unable to write $tarFile: $!";
    print T $res->content;
    close T;

    my $fileType = `file -b $tarFile`;
    chomp $fileType;
    die "Urgh: $tarFile is not a tar file '$fileType'" unless $fileType =~ /tar/;

    my $tar = new Archive::Tar($tarFile, 1);
    my %files;
    for my $fn ($tar->list_files(['name'])) {
	my ($prefix, $f) = split '/', $fn;
	$files{$f} = $fn;
    }
    my $version = $tar->get_content($files{version});
    $version =~ s/\s+$//;
    unlink $tarFile;

    return ($version, $version);    
}

sub getFirmwareVersionOld {
    my ($self, $ip) = @_;

    my $req = GET "http://$ip/upgrade.cgi";
    
    my $res = $self->req($req);
    die "Unable to get current firmware version: ".$res->status_line
	if $res->is_error;
    
    my ($version, $ver) = 
	$res->content =~ /<td[^>]*>(?:&nbsp;)?([WLPNX]S5\.(ar2313\.v[\d\.]+[^<]*))<\/td>/;

    die "Unable to determine current version" unless $version;
    
    return ($version, $ver);
}

sub getFirmwareVersion($) {
    my ($self, $ip) = @_;

    return $self->getFirmwareVersionM($ip);

#    return $self->getFirmwareVersionOld($ip);
}


42;
       
